from django.contrib.auth.forms import UserChangeForm as BaseUserChangeForm
from django.contrib.auth.forms import UserCreationForm as BaseUserCreationForm

from .models import User


class UserCreationForm(BaseUserCreationForm):
    class Meta:
        model = User
        fields = ("email", "name")


class UserChangeForm(BaseUserChangeForm):
    class Meta:
        model = User
        fields = "__all__"



# Generated by Django 3.2.7 on 2021-09-21 22:31

from django.db import migrations, models
import django.utils.timezone
import user_email.models
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('auth', '0011_update_proxy_permissions'),
    ]

    operations = [
        migrations.CreateModel(
            name='User',
            fields=[
                ('password', models.CharField(max_length=128, verbose_name='password')),
                ('last_login', models.DateTimeField(blank=True, null=True, verbose_name='last login')),
                ('is_superuser', models.BooleanField(default=False, help_text='Designates that this user has all permissions without explicitly assigning them.', verbose_name='superuser status')),
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('email', models.EmailField(blank=True, max_length=254, null=True, unique=True)),
                ('name', models.CharField(max_length=60)),
                ('is_active', models.BooleanField(db_index=True, default=False)),
                ('is_staff', models.BooleanField(default=False)),
                ('date_joined', models.DateTimeField(db_index=True, default=django.utils.timezone.now)),
                ('groups', models.ManyToManyField(blank=True, help_text='The groups this user belongs to. A user will get all permissions granted to each of their groups.', related_name='user_set', related_query_name='user', to='auth.Group', verbose_name='groups')),
                ('user_permissions', models.ManyToManyField(blank=True, help_text='Specific permissions for this user.', related_name='user_set', related_query_name='user', to='auth.Permission', verbose_name='user permissions')),
            ],
            managers=[
                ('objects', user_email.models.UserManager()),
            ],
        ),
        migrations.AddConstraint(
            model_name='user',
            constraint=models.UniqueConstraint(condition=models.Q(('is_active', True)), fields=('email',), name='unique_active_email'),
        ),
    ]



import faker
from django.db.utils import IntegrityError
from django.test import TransactionTestCase

from .factory import User, UserFactory


class UserManagerTestCase(TransactionTestCase):
    fields = ("is_active", "is_staff", "is_superuser")

    def test_create_user(self):
        for case in (None, False, True):
            with self.subTest(case=case):
                extra_fields = {"is_active": case, "is_staff": case, "is_superuser": case} if type(case) is bool else {}
                instance = UserFactory.build(**extra_fields)
                user = User.objects.create_user(instance.name, instance.email, "password", **extra_fields)

                self.assertEqual(user.name, instance.name)
                self.assertEqual(user.email, instance.email)
                for field in self.fields:
                    self.assertEqual(getattr(user, field), bool(case))

    def test_create_user_without_email_raises_exception(self):
        self.assertRaises(
            ValueError,
            lambda _: User.objects.create_user("Bob", "", "password", **{field: True for field in self.fields}),
            "Users must have an email address.",
        )

    def test_create_superuser(self):
        for case in (None, True):
            with self.subTest(case=case):
                extra_fields = {"is_active": True, "is_staff": True, "is_superuser": True}
                instance = UserFactory.build(**extra_fields)
                user = User.objects.create_superuser(
                    instance.name, instance.email, "password", **extra_fields if case else {}
                )

                self.assertEqual(user.name, instance.name)
                self.assertEqual(user.email, instance.email)
                for field in self.fields:
                    self.assertEqual(getattr(user, field), True)

    def test_create_superuser_with_false_fields(self):
        f = faker.Faker()

        for field in self.fields:
            with self.subTest(field=field):
                self.assertRaises(
                    ValueError,
                    lambda _: User.objects.create_superuser(
                        f.name(),
                        f.email(),
                        "password",
                        **{_field: False if field == _field else True for _field in self.fields},
                    ),
                    f"Superuser must have {field}=True.",
                )

    def test_user_unique_constraint(self):
        user = UserFactory()

        with self.assertRaises(IntegrityError):
            User.objects.create_user(user.name, user.email, "password", **{field: True for field in self.fields})



import factory

from .models import User


class UserFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = User

    name = factory.Faker("name")
    email = factory.Faker("email")
    is_active = True



from django.contrib import admin
from django.contrib.auth.admin import UserAdmin as BaseUserAdmin

from .forms import UserChangeForm, UserCreationForm
from .models import User


@admin.register(User)
class UserAdmin(BaseUserAdmin):
    form = UserChangeForm
    add_form = UserCreationForm

    date_hierarchy = "date_joined"
    list_display = ("name", "email", "date_joined", "is_active", "is_staff", "is_superuser")
    list_filter = ("is_active", "is_staff", "is_superuser", "date_joined", "groups")
    fieldsets = (
        (None, {"fields": ("name", "email", "password")}),
        ("Permissions", {"fields": ("is_active", "is_staff", "is_superuser", "groups", "user_permissions")}),
        ("Important dates", {"fields": ("date_joined", "last_login")}),
    )
    # add_fieldsets is not a standard ModelAdmin attribute. UserAdmin
    # overrides get_fieldsets to use this attribute when creating a user.
    add_fieldsets = ((None, {"classes": ("wide",), "fields": ("name", "email", "password1", "password2")}),)
    readonly_fields = ("date_joined", "last_login")
    search_fields = ("email", "name")
    ordering = ("-date_joined",)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['user_email', 'user_email.migrations']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'django-user-email',
    'version': '1.0.1',
    'description': 'Custom, simple Django User model with email as username',
    'long_description': '<h1 align="center">\n  django-user-email\n</h1>\n\n<p align="center">\n  <a href="https://github.com/khasbilegt/django-user-email/">\n    <img src="https://img.shields.io/github/workflow/status/khasbilegt/django-user-email/test?label=CI&logo=github&style=for-the-badge" alt="ci status">\n  </a>\n  <a href="https://pypi.org/project/django-user-email/">\n    <img src="https://img.shields.io/pypi/v/django-user-email?style=for-the-badge" alt="pypi link">\n  </a>\n  <a href="https://codecov.io/github/khasbilegt/django-user-email">\n    <img src="https://img.shields.io/codecov/c/github/khasbilegt/django-user-email?logo=codecov&style=for-the-badge" alt="codecov">\n  </a>\n  <br>\n  <a>\n    <img src="https://img.shields.io/pypi/pyversions/django-user-email?logo=python&style=for-the-badge" alt="supported python versions">\n  </a>\n  <a>\n    <img src="https://img.shields.io/pypi/djversions/django-user-email?logo=django&style=for-the-badge" alt="supported django versions">\n  </a>\n</p>\n\n<p align="center">\n  <a href="#installation">Installation</a> •\n  <a href="#contributing">Contributing</a> •\n  <a href="#license">License</a>\n</p>\n\n<p align="center">Custom, simple Django User model with email as username</p>\n\n## Installation\n\n1. Use your preferred package manager ([pip](https://pip.pypa.io/en/stable/), [poetry](https://pypi.org/project/poetry/), [pipenv](https://pypi.org/project/pipenv/)) to install the package. For example:\n\n```bash\n$ poetry install django-user-email\n```\n\n2. Then register \'user_email\', in the \'INSTALLED_APPS\' section of your project\'s settings.\n\n```python\n# settings.py\n...\n\nINSTALLED_APPS = (\n    ...\n    \'user_email\',\n)\n\n...\n```\n\n3. Set AUTH_USER_MODEL - Since it\'s a custom User model Django needs to know the path of the model\n\n```bash\n# settings.py\n...\n\nAUTH_USER_MODEL = \'user_email.User\'\n\n...\n```\n\n## Contributing\n\nPull requests are welcome. For major changes, please open an issue first to discuss what you would like to change.\n\nPlease make sure to update tests as appropriate.\n\n## License\n\n[MIT License](https://choosealicense.com/licenses/mit/)\n',
    'author': 'Khasbilegt.TS',
    'author_email': 'khasbilegt.ts@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/khasbilegt/django-user-email',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.6.2,<4.0.0',
}


setup(**setup_kwargs)



import uuid

from django.contrib.auth.models import (
    AbstractBaseUser,
    BaseUserManager,
    PermissionsMixin,
)
from django.db import models
from django.utils import timezone


class UserManager(BaseUserManager):
    use_in_migrations = True

    def _create_user(self, name, email, password, **extra_fields):
        user = self.model(email=self.normalize_email(email), name=name, **extra_fields)
        user.set_password(password)
        user.save(using=self._db)
        return user

    def create_user(self, name, email=None, password=None, **extra_fields):
        extra_fields.setdefault("is_active", False)
        extra_fields.setdefault("is_staff", False)
        extra_fields.setdefault("is_superuser", False)
        return self._create_user(name, email, password, **extra_fields)

    def create_superuser(self, name, email=None, password=None, **extra_fields):
        extra_fields.setdefault("is_active", True)
        extra_fields.setdefault("is_staff", True)
        extra_fields.setdefault("is_superuser", True)

        if extra_fields.get("is_active") is not True:
            raise ValueError("Superuser must have is_active=True.")
        if extra_fields.get("is_staff") is not True:
            raise ValueError("Superuser must have is_staff=True.")
        if extra_fields.get("is_superuser") is not True:
            raise ValueError("Superuser must have is_superuser=True.")

        return self._create_user(name, email, password, **extra_fields)


class User(AbstractBaseUser, PermissionsMixin):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    email = models.EmailField(blank=True, null=True, unique=True)
    name = models.CharField(max_length=60)
    is_active = models.BooleanField(default=False, db_index=True)
    is_staff = models.BooleanField(default=False)
    date_joined = models.DateTimeField(default=timezone.now, db_index=True)

    objects = UserManager()

    USERNAME_FIELD = "email"
    REQUIRED_FIELDS = ["name"]

    class Meta:
        constraints = [
            models.UniqueConstraint(name="unique_active_email", fields=["email"], condition=models.Q(is_active=True)),
        ]

    def __str__(self):
        return f"{self.name} ({self.email})"
